package tw.com.lig.sdk.examples

import android.Manifest
import android.content.res.Resources
import android.graphics.*
import android.os.Bundle
import android.util.DisplayMetrics
import android.util.Log
import android.view.SurfaceHolder
import android.view.SurfaceView
import android.view.Window
import android.view.WindowManager
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import kotlinx.android.synthetic.main.activity_main.*
import tw.com.lig.sdk.lightsdk.Light
import tw.com.lig.sdk.lightsdk.LightSearcher
import tw.com.lig.sdk.lightsdk.LightSearchingCallBack

class MainActivity : AppCompatActivity(), LightSearchingCallBack {

    companion object {
        const val PERMISSION_REQ_CODE = 0x101

        val PERMISSION_LIST = arrayOf(
            Manifest.permission.CAMERA,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.READ_PHONE_STATE,
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION
        )
    }

    private var surfaceView: SurfaceView? = null
    private var surfaceHolder: SurfaceHolder? = null
    private var lightBitmap: Bitmap? = null
    private var screenWidth = 0
    private var screenHeight = 0
    private val lightPaint = Paint()
    private val lightText = Paint()


    override fun receiveResults(status: Int, lights: MutableList<Light>?) {
        runOnUiThread {
            drawView(lights!!)
        }
    }

    override fun onResume() {
        super.onResume()
        checkPermission()
    }

    override fun onPause() {
        super.onPause()
        LightSearcher.stop()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        window.setFlags(
            WindowManager.LayoutParams.FLAG_FULLSCREEN,
            WindowManager.LayoutParams.FLAG_FULLSCREEN
        )
        setContentView(R.layout.activity_main)

        initScreenSize()
        initView()
    }

    private fun initScreenSize() {
        val metric = DisplayMetrics()
        windowManager.defaultDisplay.getMetrics(metric)
        screenWidth = metric.widthPixels
        screenHeight = metric.heightPixels
    }

    private fun initView() {
        surfaceView = findViewById(R.id.surfaceview)
        surfaceView!!.setZOrderOnTop(true)
        surfaceView!!.holder.setFormat(PixelFormat.TRANSLUCENT)
        surfaceHolder = surfaceView!!.holder
        lightBitmap = BitmapFactory.decodeResource(resources, R.drawable.light)
        lightText.color = Color.WHITE
        lightText.textSize = dp2px(20)
    }

    @Synchronized
    private fun drawView(lights: List<Light>) {
        if (lights.isNotEmpty()) {
            val canvas = surfaceHolder!!.lockCanvas() ?: return
            // Clear canvas
            canvas.drawColor(Color.TRANSPARENT, PorterDuff.Mode.CLEAR)

            for (light in lights) {
                if (!light.valid) {
                    continue
                }

                val screenX = light.screenX * surfaceView!!.width
                val screenY = light.screenY * surfaceView!!.height

                val bitmap = lightBitmap!!
                val labelWidth = bitmap.width
                val labelHeight = bitmap.height
                val rect = RectF(0.0f, 0.0f, 0.0f, 0.0f)
                rect.set(screenX, screenY, labelWidth + screenX, labelHeight + screenY)
                canvas.drawBitmap(bitmap, null, rect, lightPaint)
                canvas.drawText(
                    "Light:" + light.id.toString(), screenX
                    , labelWidth + screenY + lightText.textSize, lightText
                )

                val str = "Location: " + light.x + "," + light.y + "," + light.z
                txtLocation.text = str

                break
            }
            surfaceHolder!!.unlockCanvasAndPost(canvas)
        }
    }

    private fun dp2px(dpValue: Int): Float {
        val scale =
            Resources.getSystem().displayMetrics.density
        return dpValue * scale + 0.5f
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>, grantResults: IntArray
    ) {
        when (requestCode) {
            PERMISSION_REQ_CODE -> {
                if (grantResults.isNotEmpty() && PermissionUtils.isAllGranted(grantResults)) {
                    LightSearcher.searchLights(this@MainActivity, this@MainActivity)
                } else {
                    AlertDialog.Builder(this@MainActivity)
                        .setTitle("WARNING")
                        .setMessage("You must allow all permission.")
                        .setPositiveButton(getString(android.R.string.ok), null)
                        .show()
                }
                return
            }
            else -> {
            }
        }
    }

    private fun checkPermission() {
        with(PermissionUtils) {
            if (isGranted(this@MainActivity, Manifest.permission.CAMERA) &&
                isGranted(this@MainActivity, Manifest.permission.WRITE_EXTERNAL_STORAGE) &&
                isGranted(this@MainActivity, Manifest.permission.READ_EXTERNAL_STORAGE) &&
                isGranted(this@MainActivity, Manifest.permission.READ_PHONE_STATE) &&
                isGranted(this@MainActivity, Manifest.permission.ACCESS_COARSE_LOCATION) &&
                isGranted(this@MainActivity, Manifest.permission.ACCESS_FINE_LOCATION)
            ) {
                LightSearcher.searchLights(this@MainActivity, this@MainActivity)
            } else {
                ActivityCompat.requestPermissions(
                    this@MainActivity,
                    PERMISSION_LIST,
                    PERMISSION_REQ_CODE
                )
            }
        }
    }
}
